@@ -2,8 +2,6 @@ package JSON::SL::Tuba;
 use strict;
 use warnings;
 use JSON::SL;
-use Data::Dumper;
-use Log::Fu;
 use base qw(Exporter);
 our @EXPORT;
 
@@ -109,10 +107,6 @@ foreach (['key', '#'],
     }
 }
 
-sub on_any {
-    log_warn("Unhandled..");
-}
-
 1;
 
 __END__
